package cn.com.qmth.examcloud.tool.service.batch_delete_exam_student;

import cn.com.qmth.examcloud.tool.config.SysProperty;
import cn.com.qmth.examcloud.tool.entity.TaskEntity;
import cn.com.qmth.examcloud.tool.service.CommonService;
import cn.com.qmth.examcloud.tool.service.TaskService;
import cn.com.qmth.examcloud.tool.service.batch_delete_exam_student.vo.ExamStudentExcelVO;
import cn.com.qmth.examcloud.tool.utils.HttpHelper;
import cn.com.qmth.examcloud.tool.vo.examstudent.EwExamStudentVO;
import cn.com.qmth.examcloud.tool.vo.user.User;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class BatchDeleteExamStudentTask implements TaskService {

    private final static Logger log = LoggerFactory.getLogger(BatchDeleteExamStudentTask.class);

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private CommonService commonService;

    @Override
    public void start(TaskEntity task) {
        // this.execute(loginUser, task);
    }

    /**
     * 批量删除某个考试的考生列表
     */
    public void deleteExamStudentForExcel(User loginUser) {
        String dataFilePath = "D:/home/data/delete_exam_students.xlsx";

        long startTime = System.currentTimeMillis();
        List<ExamStudentExcelVO> list = new ArrayList<>();
        try {
            EasyExcel.read(dataFilePath, ExamStudentExcelVO.class, new AnalysisEventListener<ExamStudentExcelVO>() {
                        @Override
                        public void invoke(ExamStudentExcelVO data, AnalysisContext analysisContext) {
                            list.add(data);
                        }

                        @Override
                        public void doAfterAllAnalysed(AnalysisContext analysisContext) {
                            // ignore
                        }
                    }
            ).sheet().doRead();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("Excel内容解析错误，请使用标准模板！");
        }

        log.info("待删除考生数:{}", list.size());

        int index = 0;
        for (ExamStudentExcelVO vo : list) {
            index++;
            List<EwExamStudentVO> examStudents = commonService.getExamStudentListFromEW(loginUser, vo.getExamId()
                    , null, vo.getCourseCode(), vo.getIdentityNumber());
            if (examStudents.isEmpty()) {
                log.warn("【未找到考生】examId:{} courseCode:{} identityNumber:{} index:{}", vo.getExamId(), vo.getCourseCode(), vo.getIdentityNumber(), index);
                continue;
            }

            for (EwExamStudentVO examStudent : examStudents) {
                log.info("【待删除】examId:{} courseCode:{} identityNumber:{} examStudentId:{} index:{}",
                        examStudent.getExamId(), examStudent.getCourseCode(),
                        examStudent.getIdentityNumber(), examStudent.getId(), index);
                this.deleteExamStudent(loginUser, examStudent.getId());
            }
        }

        log.info("删除完成，耗时:{}ms", System.currentTimeMillis() - startTime);
    }

    private void deleteExamStudent(User loginUser, Long examStudentId) {
        Map<String, String> headers = new HashMap<>();
        headers.put("key", loginUser.getKey());
        headers.put("token", loginUser.getToken());

        String url = loginUser.getServerUrl() + "/api/ecs_exam_work/exam_student/" + examStudentId;
        String result = HttpHelper.delete(url, headers, null);
        log.info("【删除已执行】{}", result);
    }

}
